//
//  ChatNetwork.swift
//  migamy
//
//  Created by Daniel Funes Garcia-Penche on 15/05/2017.
//  Copyright © 2017 Daniel Funes Garcia-Penche. All rights reserved.
//

import Foundation
import FirebaseDatabase
import ObjectMapper

protocol ChatDelegate {
    func onReceiveMessage(message: Message)
    func onTypingStatusChange(status: Bool)
}

class ChatNetwork {
    private var channelRef: FIRDatabaseReference
    private var channelRefHandle: FIRDatabaseHandle?
    private var typingRef: FIRDatabaseReference
    private var delegate: ChatDelegate
    
    private var localTyping = false
    var userIsTyping: Bool {
        get {
            return localTyping
        }
        set {
            localTyping = newValue
            typingRef.child(UserNetwork.currentUser().uid).setValue(newValue)
        }
    }
    
    private var localOtherTyping = false
    var otherUserIsTyping: Bool {
        get {
            return localOtherTyping
        }
        set {
            localOtherTyping = newValue
            delegate.onTypingStatusChange(status: newValue)
        }
    }
    
    init(delegate: ChatDelegate, room: Room) {
        self.delegate = delegate
        channelRef = FIRDatabase.database().reference().child("messages/\(room.id)")
        typingRef = channelRef.child("typingIndicator")
        typingRef.onDisconnectRemoveValue()
        typingRef
            .queryOrderedByValue()
            .queryEqual(toValue: true)
            .observe(.value, with: { (snapshot) in
                let onlyCurrentUserTyping = snapshot.childrenCount == 1 && self.userIsTyping
                self.otherUserIsTyping = !onlyCurrentUserTyping && snapshot.childrenCount > 0
        })
    }
    
    static func get(roomId: String, _ completion: @escaping (_ : Room?, _ : Error?) -> Void) {
        let path = "/chats/\(roomId)"
        
        let reference = FIRDatabase.database().reference()
        reference
            .child(path)
            .observe(.value, with: { (snapshot) in
                guard let json = snapshot.value as? [String: Any] else {
                    completion(nil, nil)
                    return
                }
                guard let room = Mapper<Room>().map(JSON: json) else {
                    completion(nil, nil)
                    return
                }
                reference.removeAllObservers()
                completion(room, nil)
            })
    }
    
    static func getRooms(_ completion: @escaping (_ : [Room], _ : Error?) -> Void) {
        let path = "/users/\(UserNetwork.currentUser().uid)/chats"
        let reference = FIRDatabase.database().reference()
        
        reference.child(path).observeSingleEvent(of: .value) { (snapshot: FIRDataSnapshot) in
            guard let snapshots = snapshot.value as? [String : Bool] else {
                return completion([], nil)
            }
            
            var rooms: [Room] = []
            for (roomId, _) in snapshots {
                reference.child("chats/\(roomId)").observeSingleEvent(of: .value) { (snapshot: FIRDataSnapshot) in
                    guard let json = snapshot.value as? [String: Any] else {
                        return
                    }
                    guard let room = Mapper<Room>().map(JSON: json) else {
                        return
                    }
                    
                    rooms.append(room)
                    
                    if rooms.count == snapshots.count {
                        return completion(rooms, nil)
                    }
                }
            }
        }
    }
    
    func listenMessages() {
        let messageQuery = channelRef.queryLimited(toLast:25)
        channelRefHandle = messageQuery.observe(.childAdded, with: { (snapshot) -> Void in
            guard let json = snapshot.value as? [String: Any] else {
                return
            }
            guard let message = Mapper<Message>().map(JSON: json) else {
                return
            }
            self.delegate.onReceiveMessage(message: message)
        })
    }
    
    func sendMessage(room: Room, message: Message) {
        userIsTyping = false
        let itemRef = channelRef.childByAutoId()
        itemRef.setValue(message.toJSON())
    }
}
