//
//  AvailabilityViewController.swift
//  Gaming
//
//  Created by Daniel Funes Garcia-Penche on 01/05/2017.
//  Copyright © 2017 Daniel Funes Garcia-Penche. All rights reserved.
//

import UIKit
import JTAppleCalendar

class NewBookViewController: UIViewController {

    @IBOutlet weak var calendarView: JTAppleCalendarView!
    @IBOutlet weak var startDateLabel: UILabel!
    @IBOutlet weak var endDateLabel: UILabel!
    @IBOutlet weak var footerView: UIView!
    
    var product: Product
    var startAt: Date!
    var endAt: Date!
    
    var itemSize = Double()
    var cellHeight = CGFloat()
    
    let formatter: DateFormatter = DateFormatter()
    let monthFormatter: DateFormatter = DateFormatter()
    
    var totalPriceLabel: UILabel!
    
    init(product: Product) {
        self.product = product
        super.init(nibName: "NewBookViewController", bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewWillAppear(_ animated: Bool) {
        
        navigationController?.navigationBar.tintColor = UIColor.darkGray
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.calendarDidLoad()
        automaticallyAdjustsScrollViewInsets = false
        startDateLabel.textColor = UIColor.blue
        endDateLabel.textColor = UIColor.blue
        calendarView.backgroundColor = UIColor.background
        
        
        let footer = Bundle.main.loadNibNamed("ProductBottomView", owner: self)?.first as! ProductBottomView
        footer.frame = CGRect(
            x: 0,
            y: 0,
            width: view.frame.width,
            height: 60
        )
        
        footer.loadView(product: product)
        self.totalPriceLabel = footer.priceLabel
        footer.bookButton.addTarget(self, action: #selector(onPressButton), for: .touchUpInside)
        footerView.addSubview(footer)
        
        if product.user.isEmpty {
            product.user = UserNetwork.currentUser().uid
        }
    }
    
    func onPressButton() {
        guard startAt != nil && endAt != nil else {
            return
        }
        let book = Book(product: product, startAt: startAt, endAt: endAt)
        BookNetwork.createBook(book: book) { (book, error) in
            if let book = book {
                self.present(ProtectedViewController(BookCreatedViewController(book: book), isModal: true), animated: true, completion: nil)
                self.navigationController?.popToRootViewController(animated: false)
            }
        }
    }
}

extension NewBookViewController {
    func calendarDidLoad() {
        formatter.dateFormat = "dd/MM/yyyy"
        formatter.timeZone = Calendar.current.timeZone
        formatter.locale = Calendar.current.locale
        
        monthFormatter.dateFormat = "MMMM yyyy"
        
        calendarView.calendarDelegate = self
        calendarView.calendarDataSource = self
        
        calendarView.register(
            UINib(nibName:"CalendarViewCell", bundle: nil),
            forCellWithReuseIdentifier: "calendarCell"
        )
        
        calendarView.register(
            UINib(nibName:"CalendarHeaderViewCell", bundle: nil),
            forSupplementaryViewOfKind: "UICollectionElementKindSectionHeader",
            withReuseIdentifier: "CalendarHeaderViewCell"
        )
        
        self.setupCalendarView()
    }
    
    func setupCalendarView() {
        let screenWidth = self.view.frame.size.width
        itemSize = Double(screenWidth) / 7
        cellHeight = CGFloat(itemSize - 5)
        calendarView.cellSize = CGFloat(itemSize)
        calendarView.minimumLineSpacing = 0
        calendarView.minimumInteritemSpacing = 0
        calendarView.allowsMultipleSelection  = true
        calendarView.isRangeSelectionUsed = true
        calendarView.scrollDirection = .vertical
        calendarView.scrollingMode = .none
    }
}


extension NewBookViewController: JTAppleCalendarViewDelegate {
    func calendar(_ calendar: JTAppleCalendarView, didSelectDate date: Date, cell: JTAppleCell?, cellState: CellState) {
        endAt = date
        calendarView.deselectAllDates()
        
        if startAt == nil {
            startAt = date
            calendarView.selectDates(from: date, to: date,  triggerSelectionDelegate: false, keepSelectionIfMultiSelectionAllowed: false)
            
        } else {
            calendarView.selectDates(from: startAt, to: date,  triggerSelectionDelegate: false, keepSelectionIfMultiSelectionAllowed: false)
        }
        
        startDateLabel.text = formatter.string(from: calendarView.selectedDates.first!)
        endDateLabel.text = formatter.string(from: calendarView.selectedDates.last!)
        
        let price = product.getPrice(forDays: calendarView.selectedDates.count - 1)
        if price > 0 {
            totalPriceLabel.text = "Total price".localize(wildcard: "\(price)")
        }
        calendarView.reloadData()
    }
    
    func calendar(_ calendar: JTAppleCalendarView, headerViewForDateRange range: (start: Date, end: Date), at indexPath: IndexPath) -> JTAppleCollectionReusableView {
        let cell = calendar.dequeueReusableJTAppleSupplementaryView(withReuseIdentifier: "CalendarHeaderViewCell", for: indexPath) as! CalendarHeaderViewCell
        cell.title.text = monthFormatter.string(from: range.start)
    
        return cell
    }
    
    func calendar(_ calendar: JTAppleCalendarView, shouldSelectDate date: Date, cell: JTAppleCell, cellState: CellState) -> Bool {
        return startAt == nil || startAt < date
    }
    
    func calendar(_ calendar: JTAppleCalendarView, cellForItemAt date: Date, cellState: CellState, indexPath: IndexPath) -> JTAppleCell {
        let cell: CalendarViewCell = calendar.dequeueReusableJTAppleCell(withReuseIdentifier: "calendarCell", for: indexPath) as! CalendarViewCell
        cell.dateLabel.text = cellState.text
        handleCellColor(cell: cell, cellState: cellState, date: date)
        handleCellTextColor(cell: cell, cellState: cellState, date: date)
        return cell
    }
    
    
    func calendarSizeForMonths(_ calendar: JTAppleCalendarView?) -> MonthSize? {
        return MonthSize(defaultSize: CGFloat(self.cellHeight))
    }
    
    private func handleCellColor(cell: CalendarViewCell, cellState: CellState, date: Date) {
        cell.unselect()
        guard startAt != nil && endAt != nil && cellState.dateBelongsTo == .thisMonth else {
            return
        }
        
        if date == startAt && date == endAt {
            cell.select()
            return
        }
        
        if date == startAt {
            cell.selectAsFirst()
            return
        }
        
        if date == endAt {
            cell.selectAsLast()
            return
        }
            
        if date < endAt && date > startAt {
            cell.selectAsMiddle()
        }
    }
    
    private func handleCellTextColor(cell: CalendarViewCell, cellState: CellState, date: Date) {
        if cellState.dateBelongsTo != .thisMonth {
            cell.dateLabel.textColor = UIColor.white
            cell.isUserInteractionEnabled = false
            return
        }
        
        cell.isUserInteractionEnabled = true
        
        guard startAt != nil && endAt != nil else {
            return
        }
        
        cell.dateLabel.textColor = date == startAt || date == endAt || (date < endAt && date > startAt)
            ? UIColor.white
            : UIColor.black
    }
}


extension NewBookViewController: JTAppleCalendarViewDataSource{
    func configureCalendar(_ calendar: JTAppleCalendarView) -> ConfigurationParameters {
        let startDate = Date()
        let endDate = Calendar.current.date(byAdding: .year, value: 1, to: startDate)!
        
        let parameters = ConfigurationParameters(
            startDate: startDate,
            endDate: endDate,
            calendar: Calendar.current,
            generateInDates: .forAllMonths,
            generateOutDates: .off
        )
        
        return parameters
    }
}
