//
//  RoomDetailViewController.swift
//  migamy
//
//  Created by Daniel Funes Garcia-Penche on 12/05/2017.
//  Copyright © 2017 Daniel Funes Garcia-Penche. All rights reserved.
//

import UIKit
import Firebase
import JSQMessagesViewController

class RoomDetailViewController: JSQMessagesViewController {
    
    var room: Room! {
        didSet {
            title = room.title
            chatNetwork = ChatNetwork(delegate: self, room: room)
            chatNetwork.listenMessages()
            headerView.statusLabel.text = room.status.localize()
            headerView.infoButton.addTarget(self, action: #selector(onTapInfoButton), for: .touchUpInside)
        }
    }
    
    var roomLoaded: Room {
        get { return room }
        set { room = newValue }
    }
    
    var chatNetwork: ChatNetwork!
    var book: Book!
    var messages: [JSQMessage] = []
    var headerView: ChatHeaderTableViewCell!
    
    lazy var outgoingBubbleImageView: JSQMessagesBubbleImage = self.setupOutgoingBubble()
    lazy var incomingBubbleImageView: JSQMessagesBubbleImage = self.setupIncomingBubble()
    
    func loadRoom(room: Room) {
        self.room = room
    }
    
    init(book: Book) {
        super.init(nibName: "RoomDetailViewController", bundle: nil)
        self.book = book
    }
    
    init(room: Room) {
        super.init(nibName: "RoomDetailViewController", bundle: nil)
        self.room = room
        BookNetwork.get(status: room.status, bookId: room.book) { (book, error) in
            if let book = book {
                self.book = book
            }
        }
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        senderDisplayName = UserNetwork.currentUser().email
        senderId = UserNetwork.currentUser().uid
        inputToolbar.contentView.leftBarButtonItem = nil
        
        collectionView!.collectionViewLayout.incomingAvatarViewSize = CGSize.zero
        collectionView!.collectionViewLayout.outgoingAvatarViewSize = CGSize.zero
        addInfoViewToTop()
        
        guard room != nil else {
            return ChatNetwork.get(roomId: book.id, { (room, error) in
                if let room =  room {
                    self.roomLoaded = room
                }
            })
        }
        
        roomLoaded = room
    }
    
    func addInfoViewToTop() {
        topContentAdditionalInset = 40
        headerView = Bundle.main.loadNibNamed("ChatHeaderTableViewCell", owner: self, options: nil)?.first as! ChatHeaderTableViewCell
        headerView.frame = CGRect(x: 0, y: 60, width: UIScreen.main.bounds.width, height: 40)
        view.addSubview(headerView)
    }
    
    func onTapInfoButton() {
        navigationController?.pushViewController(BookDetailViewController(book: book), animated: true)
    }
}

extension RoomDetailViewController: ChatDelegate {
    func onReceiveMessage(message: Message) {
        addMessage(withId: message.senderId, name: message.senderName, text: message.text)
        self.finishReceivingMessage()
    }
    
    func onTypingStatusChange(status: Bool) {
        showTypingIndicator = status
        scrollToBottom(animated: true)
    }
}


extension RoomDetailViewController {
    
    override func collectionView(_ collectionView: JSQMessagesCollectionView!, messageDataForItemAt indexPath: IndexPath!) -> JSQMessageData! {
        return messages[indexPath.row]
    }
    
    override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return messages.count
    }
    
    override func collectionView(_ collectionView: JSQMessagesCollectionView!, messageBubbleImageDataForItemAt indexPath: IndexPath!) -> JSQMessageBubbleImageDataSource! {
        let message = messages[indexPath.item]
        if message.senderId == senderId {
            return outgoingBubbleImageView
        } else {
            return incomingBubbleImageView
        }
    }
    
    override func collectionView(_ collectionView: JSQMessagesCollectionView!, avatarImageDataForItemAt indexPath: IndexPath!) -> JSQMessageAvatarImageDataSource! {
        return nil
    }
    
    override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = super.collectionView(collectionView, cellForItemAt: indexPath) as! JSQMessagesCollectionViewCell
        let message = messages[indexPath.item]
        
        if message.senderId == senderId {
            cell.textView?.textColor = UIColor.white
        } else {
            cell.textView?.textColor = UIColor.black
        }
        return cell
    }
    
    override func textViewDidChange(_ textView: UITextView) {
        super.textViewDidChange(textView)
        chatNetwork.userIsTyping = textView.text != ""
    }
    
    override func didPressSend(_ button: UIButton!, withMessageText text: String!, senderId: String!, senderDisplayName: String!, date: Date!) {
        let message = Message(JSON: [
            "senderId": senderId,
            "senderName": senderDisplayName,
            "text": text
        ])
        
        chatNetwork.sendMessage(room: room, message: message!)
        
        JSQSystemSoundPlayer.jsq_playMessageSentSound()
        finishSendingMessage()
    }
}



extension RoomDetailViewController {
    func setupOutgoingBubble() -> JSQMessagesBubbleImage {
        let bubbleImageFactory = JSQMessagesBubbleImageFactory()
        return bubbleImageFactory!.outgoingMessagesBubbleImage(with: UIColor.blue)
    }
    
    func setupIncomingBubble() -> JSQMessagesBubbleImage {
        let bubbleImageFactory = JSQMessagesBubbleImageFactory()
        return bubbleImageFactory!.incomingMessagesBubbleImage(with: UIColor.jsq_messageBubbleLightGray())
    }
    
    func addMessage(withId id: String, name: String, text: String) {
        guard !text.isEmpty else {
            return
        }
        if let message = JSQMessage(senderId: id, displayName: name, text: text) {
            messages.append(message)
        }
    }
}
