//
//  SignUpViewController.swift
//  migamy
//
//  Created by Daniel Funes Garcia-Penche on 12/05/2017.
//  Copyright © 2017 Daniel Funes Garcia-Penche. All rights reserved.
//

import UIKit

class SignUpViewController: MainViewController {
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var signupButton: ActionButton!
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var subTitleLabel: UILabel!
    @IBOutlet weak var avatarButton: UIButton!

    var nameTextField: UITextField!
    var passwordTextField: UITextField!
    var emailTextField: UITextField!
    var latitude: Double?
    var longitude: Double?

    var currentTextField: UITextField?
    let picker = UIImagePickerController()
    var locationManager: MainLocationManager!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.uiImagePickerExtensionDidLoad()
        
        locationManager = MainLocationManager(delegate: self)
        tableView.delegate = self
        tableView.dataSource = self
        tableView.separatorStyle = .none
        tableView.allowsSelection = false
        tableView.keyboardDismissMode = .interactive
        tableView.register(UINib(nibName: "InputTextTableViewCell", bundle: nil), forCellReuseIdentifier: "cell")
        
        avatarButton.layer.cornerRadius = avatarButton.frame.height / 2
        signupButton.isUserInteractionEnabled = false
        signupButton.setTitle("Create account".localize(), for: .normal)
        
        titleLabel.text = "Create account".localize()
        subTitleLabel.text = "Fill all data".localize()
        
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: NSNotification.Name.UIKeyboardWillShow, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: NSNotification.Name.UIKeyboardWillHide, object: nil)
    }
    
    func keyboardWillShow(notification: NSNotification) {
        guard let keyboardSize = (notification.userInfo?[UIKeyboardFrameBeginUserInfoKey] as? NSValue)?.cgRectValue else {
            return
        }
        self.tableView.contentInset = UIEdgeInsets(top: self.tableView.contentInset.top, left: 0, bottom: keyboardSize.height, right: 0)
        
        if self.currentTextField != nil {
            let activeTextFieldRect = self.currentTextField?.superview?.superview?.frame
            self.tableView.scrollRectToVisible(activeTextFieldRect!, animated:true)
        }
    }
    
    func keyboardWillHide(notification: NSNotification) {
        let contentInsets = UIEdgeInsets(top: self.tableView.contentInset.top, left: 0, bottom: 0, right: 0)
        self.tableView.contentInset = contentInsets
        self.currentTextField = nil
    }
    
    
    override func viewDidAppear(_ animated: Bool) {
        locationManager.determineMyCurrentLocation()
    }
    
    @IBAction func onTapPhoto(sender: UIButton) {
        let actionSheetController: UIAlertController = UIAlertController(
            title: "Upload photo".localize(),
            message: "Take a good photography!".localize(),
            preferredStyle: .actionSheet
        )
        
        let takePictureAction: UIAlertAction = UIAlertAction(title: "Take photo".localize(), style: .default) { action -> Void in
            self.presentPhotoPickerAction(UIImagePickerControllerSourceType.camera)
        }
        
        let choosePictureAction: UIAlertAction = UIAlertAction(title: "Shoose one from your album".localize(), style: .default) { action -> Void in
            self.presentPhotoPickerAction(UIImagePickerControllerSourceType.photoLibrary)
        }
        
        actionSheetController.addAction(UIAlertAction(title: "Cancel".localize(), style: .cancel))
        actionSheetController.addAction(takePictureAction)
        actionSheetController.addAction(choosePictureAction)
        
        present(actionSheetController, animated: true, completion: nil)
    }
    
    @IBAction func onTapSignupButton(_ sender: ActionButton) {
        guard nameTextField.text != nil && emailTextField.text != nil && passwordTextField.text != nil else {
            return
        }
        let avatar =  avatarButton.imageView?.image == nil
            ? UIImage(named: "ic_avatar_happy")
            : avatarButton.imageView?.image
        
        let user = User(
            name: nameTextField.text!,
            email: emailTextField.text!,
            avatar: avatar!,
            latitude: latitude,
            longitude: longitude
        )
        
        UserNetwork.signUp(user: user, password: passwordTextField.text!) { (user, error) in
            if let error = error {
                print(error.description)
            }
            self.view.window!.rootViewController?.dismiss(animated: true, completion: nil)
        }
    }
}


extension SignUpViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 100
    }
}

extension SignUpViewController: UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 3
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "cell", for: indexPath) as! InputTextTableViewCell
        cell.inputTextField.delegate = self
        
        switch indexPath.row {
        case 0:
            cell.inputTextField.placeholder = "Name".localize()
            cell.inputTextField.keyboardType = .asciiCapable
            cell.imageView?.image = UIImage(named: "ic_name")
            nameTextField = cell.inputTextField
        case 1:
            cell.inputTextField.placeholder = "Email".localize()
            cell.inputTextField.keyboardType = .emailAddress
            cell.imageView?.image = UIImage(named: "ic_email")
            emailTextField = cell.inputTextField
        default:
            cell.inputTextField.placeholder = "Password".localize()
            cell.inputTextField.isSecureTextEntry = true
            cell.imageView?.image = UIImage(named: "ic_locked")
            passwordTextField = cell.inputTextField
        }

        return cell
    }
}

extension SignUpViewController: UITextFieldDelegate {
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        signupButton.isUserInteractionEnabled = true
            && !(nameTextField.text?.isEmpty)!
            && !(emailTextField.text?.isEmpty)!
            && !(passwordTextField.text?.isEmpty)!
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        currentTextField = textField
        
        if textField === self.nameTextField {
            emailTextField.becomeFirstResponder()
        } else if textField === self.emailTextField {
            passwordTextField.becomeFirstResponder()
        } else {
            view.endEditing(true)
        }
        
        return true
    }
}

extension SignUpViewController: LocationManagerDelegate {
    func onFailLocation(error: Error) {
        print(error.localizedDescription)
    }
    
    func onReceiveCoordinates(latitude: Double, longitude: Double) {
        self.latitude = latitude
        self.longitude = longitude
    }
}

extension SignUpViewController: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    func uiImagePickerExtensionDidLoad() {
        picker.delegate = self
    }
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : Any]) {
        let chosenImage = info[UIImagePickerControllerOriginalImage] as! UIImage
        avatarButton.setImage(chosenImage, for: .normal)
        dismiss(animated: true, completion: nil)
    }
    
    func imagePickerControllerDidCancel(_ picker: UIImagePickerController) {
        dismiss(animated: true, completion: nil)
    }
    
    
    func presentPhotoPickerAction(_ type: UIImagePickerControllerSourceType) {
        self.picker.allowsEditing = false
        self.picker.sourceType = type
        self.present(self.picker, animated: true, completion: nil)
    }
}

