//
//  UserNetwork.swift
//  migamy
//
//  Created by Daniel Funes Garcia-Penche on 13/05/2017.
//  Copyright © 2017 Daniel Funes Garcia-Penche. All rights reserved.
//

import Foundation
import FirebaseAuth
import Firebase
import ObjectMapper

protocol UserNetworkError: Error {
    var title: String { get }
    var description: String { get }
    var code: Int { get }
}

class UserNetwork {
    
    struct Error: UserNetworkError {
        var title: String
        var description: String
        var code: Int
        
        init(title: String, description: String, code: Int? = nil) {
            self.title = title
            self.description = description
            self.code = code ?? 0
        }
    }
    
    static func isLogged() -> Bool {
        return FIRAuth.auth()?.currentUser != nil
    }
    
    static func logout() -> Void {
        try! FIRAuth.auth()?.signOut()
        
        NotificationCenter.default.post(name: NSNotification.Name(rawValue: Constant.Event.logout), object: nil)
    }
    
    static func currentUser() -> FIRUser {
        return (FIRAuth.auth()?.currentUser)!
    }
    
    static func login(email: String, password: String, _ completion: @escaping (_ : FIRUser?, _ : Error? ) -> Void) {
        
        FIRAuth.auth()?.signIn(withEmail: email, password: password) { (user, error) in
            if error != nil {
                if let code = FIRAuthErrorCode(rawValue: (error?._code)!) {
                    switch code {
                    case .errorCodeInvalidEmail, .errorCodeWrongPassword, .errorCodeUserNotFound:
                        completion(nil, Error(title: "Error", description: "Invalid credentials".localize()))
                    default:
                        completion(nil, Error(title: "Error", description: error.debugDescription))
                    }
                    
                }
            }
            
            if let user = user {
                NotificationCenter.default.post(name: NSNotification.Name(rawValue: Constant.Event.login), object: nil)
                completion(user, nil)
            }
        }
    }
    
    static func signUp(user: User, password: String, _ completion: @escaping (_ : FIRUser?, _ : Error? ) -> Void) {
        
        FIRAuth.auth()?.createUser(withEmail: user.email, password: password, completion: { (firUser, error) in
            if let error = error {
                if let code = FIRAuthErrorCode(rawValue: error._code) {
                    switch code {
                    case .errorCodeEmailAlreadyInUse :
                        completion(nil, Error(title: "Error", description: "Duplicated email".localize()))
                    default:
                        completion(nil, Error(title: "Error", description: error.localizedDescription))
                    }
                }
            }
            
            if let firUser = firUser {
                let metaData = FIRStorageMetadata()
                metaData.contentType = "image/jpg"
                
                let photo = UIImageJPEGRepresentation(user.avatarImage, 0.8)
                
                let task = FIRStorage.storage(url:Constant.Firebase.storageUrl).reference()
                    .child("users/\(String(describing: firUser.uid))/avatar-\(NSDate()).jpg")
                    .put(photo!, metadata: metaData)
                
                task.observe(.success, handler: { snapshot in
                    let url = snapshot.metadata?.downloadURL()?.absoluteString
                    
                    FIRDatabase.database().reference()
                        .child(Constant.Firebase.Snapshot.users)
                        .child(firUser.uid)
                        .setValue([
                            "id": firUser.uid,
                            "name": user.name,
                            "email": user.email,
                            "avatar": url ?? "",
                            "latitude": user.latitude,
                            "longitude": user.longitude
                        ])
                    
                    NotificationCenter.default.post(name: NSNotification.Name(rawValue: Constant.Event.login), object: nil)
                    completion(firUser, nil)
                })
            }
        })
    }

    
    static func getUser(userId: String, _ completion: @escaping (_ : User?, _ : Error?) -> Void) {
        FIRDatabase
            .database()
            .reference()
            .child("\(Constant.Firebase.Snapshot.users)/\(userId)")
            .observeSingleEvent(of: .value) { (snapshot: FIRDataSnapshot) in
                guard let json = snapshot.value as? [String: Any] else {
                    return completion(nil, nil)
                }
                
                guard let user = Mapper<User>().map(JSON: json) else {
                    return completion(nil, nil)
                }
                
                completion(user, nil)
        }
    }
}
